@@ -1,6 +1,7 @@
 package CPAN::Reporter::Config;
-$VERSION = '0.99_07';
+$VERSION = '0.99_08';
 use strict; 
+use Config::Tiny ();
 use File::HomeDir (); 
 use File::Path (qw/mkpath/);
 use File::Spec ();
@@ -39,6 +40,129 @@ HERE
         rmdir($old) or die $!;
     }
 }
+
+#--------------------------------------------------------------------------#
+# Public
+#--------------------------------------------------------------------------#
+
+sub _configure {
+    my $config_dir = _get_config_dir();
+    my $config_file = _get_config_file();
+    
+    mkpath $config_dir if ! -d $config_dir;
+    if ( ! -d $config_dir ) {
+        $CPAN::Frontend->myprint(
+            "\nCouldn't create configuration directory '$config_dir': $!"
+        );
+        return;
+    }
+
+    my $config;
+    my $existing_options;
+    
+    # explain grade:action pairs
+    $CPAN::Frontend->myprint( _grade_action_prompt() );
+    
+    # read or create
+    if ( -f $config_file ) {
+        $CPAN::Frontend->myprint(
+            "\nFound your CPAN::Reporter config file at:\n$config_file\n"
+        );
+        $config = _open_config_file();
+        # if we can't read it, bail out
+        if ( ! $config ) {
+            $CPAN::Frontend->mywarn("\n
+                CPAN::Reporter configuration will not be changed\n");
+            return;
+        }
+        # clone what's in the config file
+        $existing_options = { %{$config->{_}} } if $config;
+        $CPAN::Frontend->myprint(
+            "\nUpdating your CPAN::Reporter configuration settings:\n"
+        );
+    }
+    else {
+        $CPAN::Frontend->myprint(
+            "\nNo CPAN::Reporter config file found; creating a new one.\n"
+        );
+        $config = Config::Tiny->new();
+    }
+    
+    my %spec = _config_spec();
+
+    for my $k ( _config_order() ) {
+        my $option_data = $spec{$k};
+        $CPAN::Frontend->myprint( "\n" . $option_data->{info}. "\n");
+        # options with defaults are mandatory
+        if ( defined $option_data->{default} ) {
+            # if we have a default, always show as a sane recommendation
+            if ( length $option_data->{default} ) {
+                $CPAN::Frontend->myprint(
+                    "(Recommended: '$option_data->{default}')\n\n"
+                );
+            }
+            # repeat until validated
+            PROMPT:
+            while ( defined ( 
+                my $answer = CPAN::Shell::colorable_makemaker_prompt(
+                    "$k?", 
+                    $existing_options->{$k} || $option_data->{default} 
+                )
+            )) {
+                if  ( ! $option_data->{validate} ||
+                        $option_data->{validate}->($k, $answer)
+                    ) {
+                    $config->{_}{$k} = $answer;
+                    last PROMPT;
+                }
+            }
+        }
+        else {
+            # only initialize options without default if
+            # answer matches non white space and validates, 
+            # otherwise reset it
+            my $answer = CPAN::Shell::colorable_makemaker_prompt( 
+                "$k?", 
+                $existing_options->{$k} || q{} 
+            ); 
+            if ( $answer =~ /\S/ ) {
+                $config->{_}{$k} = $answer;
+            }
+            else {
+                delete $config->{_}{$k};
+            }
+        }
+        # delete existing as we proceed so we know what's left
+        delete $existing_options->{$k};
+    }
+
+    # initialize remaining existing options
+    $CPAN::Frontend->myprint(
+        "\nYour CPAN::Reporter config file also contains these advanced " .
+          "options:\n\n") if keys %$existing_options;
+    for my $k ( keys %$existing_options ) {
+        $config->{_}{$k} = CPAN::Shell::colorable_makemaker_prompt( 
+            "$k?", $existing_options->{$k} 
+        ); 
+    }
+
+    $CPAN::Frontend->myprint( 
+        "\nWriting CPAN::Reporter config file to '$config_file'.\n"
+    );
+    if ( $config->write( $config_file ) ) {
+        return $config->{_};
+    }
+    else {
+        $CPAN::Frontend->mywarn( "\nError writing config file to '$config_file':" . 
+             Config::Tiny->errstr(). "\n");
+        return;
+    }
+}
+
+#--------------------------------------------------------------------------#
+# Private
+#--------------------------------------------------------------------------#
+
 #--------------------------------------------------------------------------#
 # _config_order -- determines order of interactive config.  Only items 
 # in interactive config will be written to a starter config file
@@ -53,29 +177,6 @@ sub _config_order {
     );
 }
 
-#--------------------------------------------------------------------------#
-# _grade_action_prompt -- describes grade action pairs
-#--------------------------------------------------------------------------#
-
-sub _grade_action_prompt {
-    return << 'HERE';
-
-Some of the following configuration options require one or more "grade:action"
-pairs that determine what grade-specific action to take for that option.
-These pairs should be space-separated and are processed left-to-right. See
-CPAN::Reporter documentation for more details.
-
-    GRADE   :   ACTION  ======> EXAMPLES        
-    -------     -------         --------    
-    pass        yes             default:no
-    fail        no              default:yes pass:no
-    unknown     ask/no          default:ask/no pass:yes fail:no
-    na          ask/yes         
-    default
-
-HERE
-}
-
 #--------------------------------------------------------------------------#
 # _config_spec -- returns configuration options information
 #
@@ -170,6 +271,76 @@ HERE
 
 sub _config_spec { return %option_specs }
 
+#--------------------------------------------------------------------------#
+# _get_config_dir
+#--------------------------------------------------------------------------#
+
+sub _get_config_dir {
+    return ( $^O eq 'MSWin32' )
+        ? File::Spec->catdir(File::HomeDir->my_documents, ".cpanreporter")
+        : File::Spec->catdir(File::HomeDir->my_home, ".cpanreporter") ;
+}
+
+#--------------------------------------------------------------------------#
+# _get_config_file
+#--------------------------------------------------------------------------#
+
+sub _get_config_file {
+    return File::Spec->catdir( _get_config_dir, "config.ini" );
+}
+
+#--------------------------------------------------------------------------#
+# _get_config_options
+#--------------------------------------------------------------------------#
+
+sub _get_config_options {
+    my $config = shift;
+    # extract and return valid options, with fallback to defaults
+    my %spec = CPAN::Reporter::Config::_config_spec();
+    my %active;
+    OPTION: for my $option ( keys %spec ) {
+        if ( exists $config->{_}{$option} ) {
+            my $val = $config->{_}{$option};
+            if  (   $spec{$option}{validate} &&
+                    ! $spec{$option}{validate}->($option, $val)
+                ) {
+                    $CPAN::Frontend->mywarn( "\nInvalid option '$val' in '$option'. Using default instead.\n\n" );
+                    $active{$option} = $spec{$option}{default};
+                    next OPTION;
+            }
+            $active{$option} = $val;
+        }
+        else {
+            $active{$option} = $spec{$option}{default}
+                if defined $spec{$option}{default};
+        }
+    }
+    return \%active;
+}
+
+#--------------------------------------------------------------------------#
+# _grade_action_prompt -- describes grade action pairs
+#--------------------------------------------------------------------------#
+
+sub _grade_action_prompt {
+    return << 'HERE';
+
+Some of the following configuration options require one or more "grade:action"
+pairs that determine what grade-specific action to take for that option.
+These pairs should be space-separated and are processed left-to-right. See
+CPAN::Reporter documentation for more details.
+
+    GRADE   :   ACTION  ======> EXAMPLES        
+    -------     -------         --------    
+    pass        yes             default:no
+    fail        no              default:yes pass:no
+    unknown     ask/no          default:ask/no pass:yes fail:no
+    na          ask/yes         
+    default
+
+HERE
+}
+
 #--------------------------------------------------------------------------#
 # _is_valid_action
 #--------------------------------------------------------------------------#
@@ -190,6 +361,18 @@ sub _is_valid_grade {
     return grep { $grade eq $_ } @valid_grades;
 }
 
+#--------------------------------------------------------------------------#
+# _open_config_file
+#--------------------------------------------------------------------------#
+
+sub _open_config_file {
+    my $config_file = _get_config_file();
+    my $config = Config::Tiny->read( $config_file )
+        or $CPAN::Frontend->mywarn("Couldn't read CPAN::Reporter configuration file " .
+                "'$config_file': " . Config::Tiny->errstr() . "\n");
+    return $config; 
+}
+
 #--------------------------------------------------------------------------#
 # _validate_grade_action 
 # returns hash of grade => action 
